function [vq] = TriInterp(x,y,v,xq,yq,ratio,flag)
%Input arguments
%x: lon
%y: lat
%v: input values
%xq: lon of query points
%yq: lat of query points
%ratio: ration of interval between lon and lat ,default = 1
fprintf('######################################################### \n');
fprintf('########     Zhang-Feng Ma, Hohai University      ####### \n');
fprintf('########     Earth Observatory of Singapore       ####### \n');
fprintf('########         Triangulation&Interpolation      ####### \n');
fprintf('###################   29,Sep,2021    #################### \n');
[m,n] = size(xq);
X = x(:);Y = y(:)*ratio;Z = v(:);
xq = xq(:);yq = yq(:);
if flag == 0
TRI = delaunayTriangulation(X,Y);
ID = pointLocation(TRI,xq,yq.*ratio);
Vx0 = X(TRI(ID,1));Vx1 = X(TRI(ID,2));Vx2 = X(TRI(ID,3));
Vy0 = Y(TRI(ID,1))./ratio;Vy1 = Y(TRI(ID,2))./ratio;Vy2 = Y(TRI(ID,3))./ratio;
Vz0 = Z(TRI(ID,1));Vz1 = Z(TRI(ID,2));Vz2 = Z(TRI(ID,3));
xkj = Vx1 - Vx0;
ykj = Vy1 - Vy0;
xlj = Vx2 - Vx0;
ylj = Vy2 - Vy0;
f = 1 ./ (xkj.*ylj - ykj.*xlj);
zj = Vz0;zk = Vz1;zl = Vz2;
zkj = zk - zj;
zlj = zl - zj;
a = -f.*(ykj.*zlj - zkj.*ylj);
b = -f.*(zkj.*xlj - xkj.*zlj);
c = -a.*Vx1 - b.*Vy1 + zk;
zout = a.* xq + b.* yq + c;
vq = reshape(zout,[m,n]);
else
Indice = knnsearch([X,Y],[xq,yq*ratio],'K',3);
Vx0 = X(Indice(:,1))-xq;
Vx1 = X(Indice(:,2))-xq;
Vx2 = X(Indice(:,3))-xq;
Vy0 = Y(Indice(:,1))./ratio-yq;
Vy1 = Y(Indice(:,2))./ratio-yq;
Vy2 = Y(Indice(:,3))./ratio-yq;
Vz0 = Z(Indice(:,1));
Vz1 = Z(Indice(:,2));
Vz2 = Z(Indice(:,3));
D0 = sqrt(Vx0.^2 + Vy0.^2 + Vz0.^2) + 0.01;%avoid zero values;
D1 = sqrt(Vx1.^2 + Vy1.^2 + Vz1.^2) + 0.01;%avoid zero values;
D2 = sqrt(Vx2.^2 + Vy2.^2 + Vz2.^2) + 0.01;%avoid zero values;
zout = (Vz0.* (1./D0) + Vz1.* (1./D1) +...
       Vz2.* (1./D2))...
       ./(1./D0 + 1./D1 + 1./D2);
vq = reshape(zout,[m,n]);
end
end

