function  ctSent_Interferometry(flag)
%CTSENT_INTERFEROMETRY Summary of this function goes here
%   Detailed explanation goes here
fprintf('#######################CtSent v1.1####################### \n');
fprintf('######################################################### \n');
fprintf('#################     Interferometry    ################# \n');
fprintf('######################################################### \n');
fprintf('########     Zhang-Feng Ma, Hohai University      ####### \n');
fprintf('########      Earth Observatory of Singapre       ####### \n');
fprintf('###################   29,Sep,2021    #################### \n');
if nargin < 1
    flag = 1;
end
parm = load('parms');
% filepath = load('coregpath');%for one burst
%%%%%%%%%%%%%for two or more bursts
if strcmp(parm.InterfeSuffix,'esd')
   filepath = load('esdpath');
else
   filepath = load('coregpath');
end
% filepath = load('esdpath');
%%%%%%%%%%%%
slcpath = filepath.slcpath;
ndate = length(slcpath);
masterIndex = 0;
for j = 1:ndate
    datestr = slcpath(j).date;
    if strcmp(datestr,parm.masterdate)
        masterIndex = j;
    end
end
if masterIndex == 0 
    fprintf('Master date cannot be detected and set the first acquisition as the Master reference... \n');
    masterIndex = 1;
end
info_mas = load(slcpath(masterIndex).infopath);
info_mas = info_mas.infosave;
nsubswath = length(info_mas.IW);
dem_read = readtiff(parm.DEMPATH);
dem_read.z(dem_read.z<-100) = 0;
if flag == 1
    for j = 1:nsubswath
        nburst = info_mas.IW(j).subSwath.numOfBursts;
        progressBar = CommandLineProgressBar(nburst);
        progressBar.message = ['Parallel Generateing Geo Files ',info_mas.IW(j).Swathname,': '];
        progressBar.barLength = 42;
    parfor k = 1:nburst
      [Latpath{k},Lonpath{k},Hgtpath{k},Incpath{k}] = generateGeofile(j,k,info_mas,dem_read,parm);
      progressBar.increment;
    end
    for k = 1:nburst
        geopath.Lat{j,k} = Latpath{k};
        geopath.Lon{j,k} = Lonpath{k};
        geopath.Hgt{j,k} = Hgtpath{k};
        geopath.Inc{j,k} = Incpath{k};
    end
    end
save 'geopath.mat' geopath;
end
for i = 1:ndate
    info_s = load(slcpath(i).infopath);
    info_s = info_s.infosave;
    for j = 1:nsubswath
        nburst = info_mas.IW(j).subSwath.numOfBursts;
        progressBar = CommandLineProgressBar(nburst);
        progressBar.message = ['Parallel Interferometry ',slcpath(i).date,' ',info_mas.IW(j).Swathname,': '];
        progressBar.barLength = 42;
    parfor k = 1:nburst
      [savepath{k},basepath{k},ifgdate{k}] = generateBpInSAR(i,j,k,masterIndex,info_mas,info_s,parm,slcpath,dem_read);
      progressBar.increment;
    end
    for k = 1:nburst
        intfpath(i).savepath{j,k} = savepath{k};
        intfpath(i).basepath{j,k} = basepath{k};
        intfpath(i).date = ifgdate{k};
    end
    end
end
save('intfpath.mat','intfpath');
end
function fwriteCt(savepath,savedata)
fileID = fopen(savepath,'w');
fwrite(fileID,savedata,'float');
fclose(fileID);
end
function [Latpath,Lonpath,Hgtpath,Incpath] = generateGeofile(j,k,info_mas,dem_read,parm)
fileDir  = [parm.workDir,'/GEO/',info_mas.IW(j).Swathname];
Latpath = [fileDir,'/lat.','burst',num2str(k)];
Lonpath = [fileDir,'/lon.','burst',num2str(k)];
Hgtpath = [fileDir,'/hgt.','burst',num2str(k)];
Incpath = [fileDir,'/inc.','burst',num2str(k)];
if ~(exist(Latpath,'file')&&exist(Lonpath,'file')&&exist(Hgtpath,'file')&&exist(Incpath,'file'))
[Lat,Lon,Elevation,Incidence] = ctSent_radarcode(info_mas.IW(j),k,dem_read);
fileDir  = [parm.workDir,'/GEO/',info_mas.IW(j).Swathname];
       if ~exist(fileDir,'dir')
         mkdir(fileDir);
       end
fwriteCt(Latpath,Lat);
fwriteCt(Lonpath,Lon);
fwriteCt(Hgtpath,Elevation);
fwriteCt(Incpath,Incidence);
else
end
end
function [savepath,basepath,ifgdate] = generateBpInSAR(i,j,k,masterIndex,info_mas,info_s,parm,slcpath,dem_read)
        if i == masterIndex
            fileDir  = [parm.workDir,'/SLC/',info_s.date,'/',info_mas.IW(j).Swathname];
            if ~exist(fileDir,'dir')
             mkdir(fileDir);
            end
            savepath = [fileDir,'/burst',num2str(k),'.slc'];
            basepath = [fileDir,'/burst',num2str(k),'.base'];
            if ~(exist(savepath,'file')&&exist(basepath,'file'))
            mas = readcpx(slcpath(i).savepath{j,k},info_mas.IW(j).sensor_para.samplesPerBurst);
            ifgdate = info_mas.date;
            writecpx(savepath,mas);
            fwriteCt(basepath,zeros(size(mas),'single'));
            return;
            else
                ifgdate = info_mas.date;
                return;
            end
        end
        masterBurstIndex = k;
        slaveBurstIndex = info_mas.IW(j).BurstUse(i,k);
if slaveBurstIndex
    fileDir  = [parm.workDir,'/SLC/',info_s.date,'/',info_mas.IW(j).Swathname];
       if ~exist(fileDir,'dir')
         mkdir(fileDir);
       end
    savepath= [fileDir,'/burst',num2str(k),'.slc'];
    basepath = [fileDir,'/burst',num2str(k),'.base'];
    ifgdate = info_s.date;
       if ~(exist(savepath,'file')&&exist(basepath,'file'))
     tempinfo_m = info_mas.IW(j);
     tempinfo_s = info_s.IW(j);
     tempinfo_m.orbit_para.orbit_points = define20vectors(tempinfo_m,masterBurstIndex);
     tempinfo_s.orbit_para.orbit_points = define20vectors(tempinfo_s,slaveBurstIndex);
    [Bperpmat,refPhasemat] = ctSent_computeBnormal(tempinfo_m,tempinfo_s,masterBurstIndex,dem_read);
    sla = readcpx(slcpath(i).savepath{j,k},info_mas.IW(j).sensor_para.samplesPerBurst);
    flat_topo = complex(cos(refPhasemat),sin(refPhasemat));
    intf = sla.*(flat_topo);
    ifgdate = info_s.date;
    writecpx(savepath,intf);
    fwriteCt(basepath,Bperpmat);
       else
           return;
       end
elseif slaveBurstIndex==0
    savepath= [];
    basepath = [];
    ifgdate = info_s.date;
end

end
function [orbitPoints] = define20vectors(info,burstIndex)
nv = 20;
m = length(info.orbit_para.orbit_points);
sUTC = info.subSwath.burstFirstLineTime(burstIndex);
dt = (info.orbit_para.orbit_points(m,1)-info.orbit_para.orbit_points(1,1)) / (m-1);
i0 = max(floor(((sUTC - info.orbit_para.orbit_points(1,1) )/ dt)) - nv/2 + 1, 0);
iN = min(i0 + nv - 1, m - 1);
if i0 == 0
    iN = 19;
elseif iN == m-1
    i0 = m-20;
end
orbitPoints = info.orbit_para.orbit_points(i0+1:iN+1,:);
end