function ctSent_geocoregSLC
%GEOCOREGSLC Summary of this function goes here
%   Detailed explanation goes here
fprintf('#######################CtSent v2.0####################### \n');
fprintf('######################################################### \n');
fprintf('###########     Geometrical Co-registration      ######## \n');
fprintf('######################################################### \n');
fprintf('########     Zhang-Feng Ma, Hohai University      ####### \n');
fprintf('########      Earth Observatory of Singapore       ###### \n');
fprintf('###################   29,Sep,2021    #################### \n');
parm = load('parms');
filepath = load('slcpath');
slcpath = filepath.slcpath;
ndate = length(slcpath);
masterIndex = 0;
for i = 1:ndate
    datestr = slcpath(i).date;
    if strcmp(datestr,parm.masterdate)
        masterIndex = i;
    end
end
if masterIndex == 0 
    fprintf('Master date cannot be detected and set the first acquisition as the Master reference... \n');
    masterIndex = 1;
end
info_mas = load(slcpath(masterIndex).infopath);
info_mas = info_mas.infosave;
nsubswath = length(info_mas.IW);
dem_read = readtiff(parm.DEMPATH);
% CoreNum = 4;
% parpool('local',CoreNum);
for i = 1:ndate
    info_sla = load(slcpath(i).infopath);
    info_sla = info_sla.infosave;
    for j = 1:nsubswath
        nburst = info_mas.IW(j).subSwath.numOfBursts;
        novl = nburst - 1;
     if novl == 0
       fprintf('Only one burst, Enhanced Spectral Diversity skipped...\n');
     end
progressBar = CommandLineProgressBar(nburst);
progressBar.message = ['Parallel Co-registration ',slcpath(i).date,' ',info_mas.IW(j).Swathname,': '];
progressBar.barLength = 42;
    for k = 1:nburst
        [savepath{k,1},derampdemodpath{k,1}] = resampleeachburst(masterIndex,i,j,k,slcpath,info_mas,info_sla,dem_read);
        progressBar.increment;
    end
    for k = 1:nburst
    slcpath(i).savepath{j,k} = savepath{k,1};slcpath(i).derampdemodpath{j,k} = derampdemodpath{k,1};
    end
    fclose('all');
    end
    slcpath(i).date = slcpath(i).date;
end
save('coregpath.mat','slcpath');
end
function [savepath,derampdemodpath] = resampleeachburst(masterIndex,i,j,k,slcpath,info_mas,info_sla,dem_read)
        existfile1 = strcat(slcpath(i).savepath{j,k},'.geo');
        if i == masterIndex
        savepath = slcpath(i).savepath{j,k};
        derampdemodpath = [];
            return;
        elseif exist(existfile1,'file')
        savepath  = strcat(slcpath(i).savepath{j,k},'.geo');
        derampdemodpath = strcat(slcpath(i).savepath{j,k},'.rampmod');   
            return;
        end
        masterBurstIndex = k;
        slaveBurstIndex = info_mas.IW(j).BurstUse(i,k);
        if slaveBurstIndex
%         fprintf(['    Resample ',info_mas.IW(j).Swathname,' No. %d burst ... \n'],k);  
        lines = info_sla.IW(j).subSwath.linesPerBurst;
        samples = info_sla.IW(j).subSwath.samplesPerBurst;
        slavecplx = freadbk(slcpath(i).savepath{j,k},lines,'mph');
        slavecplx = single(slavecplx(1:lines,1:samples));
        [resample_slavecplx,resample_slavedeRampphase] = ctSent_geoCoreg(info_mas.IW(j),info_sla.IW(j),...
            slavecplx,masterBurstIndex,slaveBurstIndex,dem_read);
        savepath = strcat(slcpath(i).savepath{j,k},'.geo');
        derampdemodpath = strcat(slcpath(i).savepath{j,k},'.rampmod');  
        writecpx(savepath,resample_slavecplx);
        fileID = fopen(derampdemodpath,'w');
        fwrite(fileID,resample_slavedeRampphase,'float');
        fclose(fileID);
        end
end
function [ dem ] = readtiff( path )
[elevation,R] = geotiffread(path);
info = geotiffinfo(path);
 height = info.Height; % Integer indicating the height of the image in pixels
 width = info.Width;
 %height1 = info.Height; % Integer indicating the height of the image in pixels
% width1 = info.Width;
 [rows,cols] = meshgrid(1:width,1:height);%[rows1,cols1] = meshgrid(1:height1,1:width1);
 % Getting the latitude and longitude of the points
 [lon,lat] = pix2map(info.RefMatrix, cols, rows);
 %[lon1,lat1] = pix2map(info.RefMatrix, rows1, cols1);
 %[lat,lon] = projinv(info, y,x);
%lon1=lon1';
%z=elevation;
dem.lat=flipud(lat);dem.lon=lon;
%Vq = geointerp(double(elevation),R,lat,lon);
dem.z = flipud(elevation);dem.R=R;
end
