function  [upslcstack,loslcstack,Lon,Lat,Elevation] = ctSent_multiplereadOvl(swathID,ovlID)
%Please pre-determine the swath and ovl number
parm = load('parms');
filepath = load('ovlpath');
slcpath = filepath.slcpath;
ndate = length(slcpath);
masterIndex = 0;
for i = 1:ndate
    datestr = slcpath(i).date;
    if strcmp(datestr,parm.masterdate)
        masterIndex = i;
    end
end
fprintf('    Extracting overlapping region... \n');
info_mas = load(slcpath(masterIndex).infopath);
info_mas = info_mas.infosave;
dem_read = readtiff(parm.DEMPATH);
[Rect_up,Rect_lo] = getOverlappedRectangles(ovlID,info_mas.IW(swathID).subSwath);
upslcstack = zeros(Rect_up(4),Rect_up(3),ndate,'single');
loslcstack = zeros(Rect_up(4),Rect_up(3),ndate,'single');
fprintf(['    Extracting ','IW ',num2str(swathID),' No. %d overlapping region... \n'],ovlID);
[Lat,Lon,Elevation,~] = ctSent_radarcode(info_mas.IW(swathID),ovlID,dem_read);
Lat = Lat(Rect_lo(2):(Rect_lo(2)+Rect_lo(4)-1),Rect_lo(1):(Rect_lo(1)+Rect_lo(3)-1));
Lon = Lon(Rect_lo(2):(Rect_lo(2)+Rect_lo(4)-1),Rect_lo(1):(Rect_lo(1)+Rect_lo(3)-1));
Elevation = Elevation(Rect_lo(2):(Rect_lo(2)+Rect_lo(4)-1),Rect_lo(1):(Rect_lo(1)+Rect_lo(3)-1));
progressBar = CommandLineProgressBar(ndate);
progressBar.message = 'Reading Overlap Region: ';
progressBar.barLength = 42;
parfor j = 1:ndate
        info_s = load(slcpath(j).infopath);
        info_s = info_s.infosave;
        [~,refPhasemat] = ctSent_computeBnormal(info_mas.IW(swathID),info_s.IW(swathID),ovlID,dem_read);
        refPhasemat = refPhasemat(Rect_lo(2):(Rect_lo(2)+Rect_lo(4)-1),Rect_lo(1):(Rect_lo(1)+Rect_lo(3)-1));
        flat_topo = complex(cos(refPhasemat),sin(refPhasemat));
        Burst_up = readcpx(slcpath(j).ovlpath{swathID,ovlID,1},Rect_up(3),Rect_up(4));
        
        Burst_up = Burst_up.*flat_topo;
        upslcstack(:,:,j) = Burst_up;
        Burst_lo = readcpx(slcpath(j).ovlpath{swathID,ovlID,2},Rect_lo(3),Rect_lo(4));
        Burst_lo = Burst_lo.*flat_topo;
        loslcstack(:,:,j) = Burst_lo;
        fclose('all');
        progressBar.increment;
end
end
function [Rectangle1,Rectangle2]=getOverlappedRectangles(overlapIndex,subSwath)
firstValidPixelOfBurstOne = getBurstFirstValidPixel(overlapIndex,subSwath);
lastValidPixelOfBurstOne = getBurstLastValidPixel(overlapIndex,subSwath);
firstValidPixelOfBurstTwo = getBurstFirstValidPixel(overlapIndex + 1,subSwath);
lastValidPixelOfBurstTwo = getBurstLastValidPixel(overlapIndex + 1,subSwath);
firstValidPixel = max(firstValidPixelOfBurstOne, firstValidPixelOfBurstTwo);
lastValidPixel = min(lastValidPixelOfBurstOne, lastValidPixelOfBurstTwo);
x0 = firstValidPixel;
w = lastValidPixel - firstValidPixel + 1;
numOfInvalidLinesInBurstOne = subSwath.linesPerBurst -...
                subSwath.lastValidLine(overlapIndex) - 1;
numOfInvalidLinesInBurstTwo = subSwath.firstValidLine(overlapIndex + 1);
numOverlappedLines = computeBurstOverlapSize(overlapIndex,subSwath);
h = numOverlappedLines - numOfInvalidLinesInBurstOne - numOfInvalidLinesInBurstTwo;
% y0BurstOne =subSwath.linesPerBurst * (overlapIndex) - numOfInvalidLinesInBurstOne - h;
y0BurstOne =subSwath.linesPerBurst - numOfInvalidLinesInBurstOne - h;
y0BurstTwo =  numOfInvalidLinesInBurstTwo;
Rectangle1=[x0, y0BurstOne, w, h];
Rectangle2=[x0, y0BurstTwo, w, h];
end
function index=getBurstFirstValidPixel(burstIndex,subSwath)
[~,n]=size(subSwath.firstValidSample);
        for lineIdx = 1:n
            if (subSwath.firstValidSample(burstIndex,lineIdx) ~= -1) 
                index=subSwath.firstValidSample(burstIndex,lineIdx);
                break;
            end
        end
end
function index=getBurstLastValidPixel(burstIndex,subSwath)
[~,n]=size(subSwath.lastValidSample);
        for lineIdx = 1:n
            if (subSwath.lastValidSample(burstIndex,lineIdx) ~= -1) 
                index=subSwath.lastValidSample(burstIndex,lineIdx);
                break;
            end
        end
end
function numOverlappedLines = computeBurstOverlapSize(overlapIndex,subSwath)
endTime = subSwath.burstLastLineTime(overlapIndex);
startTime = subSwath.burstFirstLineTime(overlapIndex + 1);
numOverlappedLines =floor((endTime - startTime) / subSwath.azimuthTimeInterval);
end
function [DEM_ex,Ellipse] = computeDEMbox(info,burstIndex,dem_read)
longitude = info.subSwath.longitude(:,burstIndex);
lonMin = min(longitude(:));lonMax = max(longitude(:));
latitude = info.subSwath.latitude(:,burstIndex);
latMin = min(latitude(:));latMax = max(latitude(:));
LatitudeLimitLo = min(dem_read.lat(:));LatitudeLimitHi = max(dem_read.lat(:));
LongitudeLimitLo = min(dem_read.lon(:));LongitudeLimitHi = max(dem_read.lon(:));
deltaDegree = abs(dem_read.lat(1,1)-dem_read.lat(2,1));
latMin = latMin - 100*deltaDegree;latMax = latMax + 100*deltaDegree;
lonMin = lonMin - 100*deltaDegree;lonMax = lonMax + 100*deltaDegree;
[RasterLine,RasterWidth] = size(dem_read.lat);
upperLeft.y = (latMin-LatitudeLimitLo)*RasterLine/(LatitudeLimitHi-LatitudeLimitLo)+1;
upperLeft.x = (lonMin-LongitudeLimitLo)*RasterWidth/(LongitudeLimitHi-LongitudeLimitLo)+1;
lowerRight.y = (latMax-LatitudeLimitLo)*RasterLine/(LatitudeLimitHi-LatitudeLimitLo)+1;
lowerRight.x = (lonMax-LongitudeLimitLo)*RasterWidth/(LongitudeLimitHi-LongitudeLimitLo)+1;
latMinIdx = floor(upperLeft.y);
latMaxIdx = floor(lowerRight.y);
lonMinIdx = floor(upperLeft.x);
lonMaxIdx = floor(lowerRight.x);
if latMinIdx <= 0
    latMinIdx = 1;
    fprintf('Warning: DEM cannot cover the whole data region !!! \n');
end
if lonMinIdx <= 0
    lonMinIdx = 1;
    fprintf('Warning: DEM cannot cover the whole data region !!! \n');
end
if latMaxIdx > RasterLine
    latMaxIdx = RasterLine;
    fprintf('Warning: DEM cannot cover the whole data region !!! \n');
end
if lonMaxIdx > RasterWidth
    lonMaxIdx = RasterWidth;
    fprintf('Warning: DEM cannot cover the whole data region !!! \n');
end
GeoBoxlat = floor(latMin):0.1:ceil(latMax);
GeoBoxlon = floor(lonMin):0.1:ceil(lonMax);
Lat_n = length(GeoBoxlat);
Lon_n = length(GeoBoxlon);
lat =repmat(GeoBoxlat',1,Lon_n);
lon =repmat(GeoBoxlon,Lat_n,1);
lon(lon<0) = abs(lon(lon<0))+180;
heightCorrection = geoidheight(lat,lon);
heightCorrection = reshape(heightCorrection,[Lat_n Lon_n]);
lon =repmat(GeoBoxlon,Lat_n,1);
DEM_ex.lat = dem_read.lat(latMinIdx:latMaxIdx,lonMinIdx:lonMaxIdx);
DEM_ex.lon = dem_read.lon(latMinIdx:latMaxIdx,lonMinIdx:lonMaxIdx);
DEM_ex.z = dem_read.z(latMinIdx:latMaxIdx,lonMinIdx:lonMaxIdx);
EGM96correction = griddata(lat,lon,heightCorrection,DEM_ex.lat,DEM_ex.lon);
DEM_ex.z = double(DEM_ex.z)+EGM96correction;
% [m,n] = size(DEM_ex.z);
% [pixel,line] = meshgrid(1:0.1:n,1:0.1:m);
% z = ba_interp2(DEM_ex.z,pixel,line,'linear');
% lat = ba_interp2(DEM_ex.lat,pixel,line,'linear');
% lon = ba_interp2(DEM_ex.lon,pixel,line,'linear');
% DEM_ex.lat = lat;DEM_ex.lon = lon;DEM_ex.z = z;
Ellipse = BLH2XYZ(DEM_ex.lat,DEM_ex.lon,DEM_ex.z);
end
