function PhU = ctSent_setrefauto(PhU,lonlat)
[ref_lonlat,robust_ind] = DBScanForReliable(lonlat(1:5:end,:),PhU(1:5:end,:));
t = linspace(0,2*pi,114); % 114 pts
r=200;                   % change radius if you want to include more points
x = r*cos(t);
y = r*sin(t);
%
% SELECT POINTS based on LONLAT
xy=llh2local(lonlat',[ref_lonlat(1),ref_lonlat(2)])'*1000;
ind = inpolygon(xy(:,1),xy(:,2),x,y);
PhU = (PhU-repmat(nanmean(PhU(ind,:),1),size(lonlat,1),1));
end
function [ref_lonlat,bound] = DBScanForReliable(lonlat,ph)
tempcoh = edgesetTempcoh(ph,lonlat(:,1),lonlat(:,2));
[grid_lon,grid_lat,grid_value,~]=DownSamp2Grid(tempcoh,ph,lonlat,1000);
[Data_Wh, ~, ~, ~] = whiten([grid_value,grid_lon,grid_lat],0.01);
label  =  dbscan(Data_Wh,0.2,100);
labeluni = unique(label);
for i = 1:length(labeluni)
    meancoh(i) = mean(grid_value(label == labeluni(i)));
end
idx = labeluni(meancoh == max(meancoh));
ind = label==idx;
lon = grid_lon(ind);
lat = grid_lat(ind);
value = grid_value(ind);
k = boundary(double(lon),double(lat));
ref_lonlat = [lon(value == max(value)),lat(value == max(value))];
bound = [lon(k),lat(k)];
subplot(1,2,1);scatter(grid_lon,grid_lat,5,grid_value,'s','filled');colormap(jet);set(gca,'clim',[0.8 1]);
hold on;scatter(ref_lonlat(1),ref_lonlat(2),100,'o','filled');plot(lon(k),lat(k));
subplot(1,2,2);scatter(lon,lat,5,value,'s','filled');colormap(jet);set(gca,'clim',[0.8 1]);
end
function tempcoh = edgesetTempcoh(ph,X,Y)
    Indice = knnsearch([X,Y],[X,Y],'K',21);
    tempcoh = zeros(length(X),1);
    for i = 1:20
        edgeIndice(:,1) = Indice(:,1);
        edgeIndice(:,2) = Indice(:,i+1);
        temp = temporalcoherence(ph,edgeIndice);
        tempcoh = tempcoh + temp;
    end
  tempcoh = tempcoh/20; 
end
function coh = temporalcoherence(ph,edgs)
nimage = size(ph,2);
temp = zeros(size(edgs,1),1);
ph = exp(1j*ph);
for i = 1:nimage
    temp = temp + (ph(edgs(:,2),i).*conj(ph(edgs(:,1),i)));
end
coh = abs(temp)/nimage;
end
function [grid_lon,grid_lat,grid_value,grid_value1] = DownSamp2Grid(tpc,ph,lonlat,pix_size)
%this function is used to regularly downsample the InSAR time series
if nargin < 3
    pix_size = 500;
end
%read data from path 1
min_lon_common = min(lonlat(:,1));
max_lon_common = max(lonlat(:,1));
min_lat_common = min(lonlat(:,2));
max_lat_common = max(lonlat(:,2));

pix_size = km2deg(pix_size/1000);

y = min_lat_common:pix_size:max_lat_common;
x = min_lon_common:pix_size:max_lon_common;
grid_value = zeros(length(y),length(x),'single');
grid_value1 = zeros(length(y),length(x),'single');
numInGrid1 = grid_value;
% grid_la1 = grid_value;
grid_lon = grid_value;
grid_lat = grid_value;
for ii=1:length(y)-1
    for jj=1:length(x)-1
        ix1 = lonlat(:,1)>=x(jj)&lonlat(:,1)<=x(jj+1)&lonlat(:,2)>=y(ii)&lonlat(:,2)<=y(ii+1);
        if sum(ix1)~=0
            grid_value(ii,jj) = mean(tpc(ix1));
            grid_value1(ii,jj) = mean(std(ph(ix1,:)));
%             grid_la1(ii,jj) = mean(inc(ix1));
            numInGrid1(ii,jj) = sum(ix1);
        end
        grid_lon(ii,jj) = (x(jj)+x(jj+1))*0.5;
        grid_lat(ii,jj) = (y(ii)+y(ii+1))*0.5;
    end
end
grid_value=grid_value(:);
grid_value1=grid_value1(:);
ind = grid_value==0|isnan(grid_value);
grid_lon=grid_lon(:);
grid_lat=grid_lat(:);
grid_value(ind)=[];
grid_value1(ind)=[];
grid_lon(ind)=[];
grid_lat(ind)=[];
end
function [Xwh, mu, invMat, whMat] = whiten(X,epsilon)


if ~exist('epsilon','var')
    epsilon = 0.0001;
end

mu = mean(X); 
X = bsxfun(@minus, X, mu);
A = X'*X;
[V,D,notused] = svd(A);
whMat = sqrt(size(X,1)-1)*V*sqrtm(inv(D + eye(size(D))*epsilon))*V';
Xwh = X*whMat;  
invMat = pinv(whMat);

end 

